@@ -1,8 +1,6 @@
 #!/usr/bin/env python
-#
-# $Id$
-#
-# Copyright (c) 2009, Jay Loden, Giampaolo Rodola'. All rights reserved.
+
+# Copyright (c) 2009, Giampaolo Rodola'. All rights reserved.
 # Use of this source code is governed by a BSD-style license that can be
 # found in the LICENSE file.
 
@@ -11,7 +9,6 @@ A clone of 'who' command; print information about users who are
 currently logged in.
 """
 
-import sys
 from datetime import datetime
 
 import psutil
