@@ -14,8 +14,10 @@
 #include <errno.h>
 #include <stdlib.h>
 #include <mntent.h>
+#include <features.h>
 #include <utmp.h>
 #include <sched.h>
+#include <linux/version.h>
 #include <sys/syscall.h>
 #include <sys/sysinfo.h>
 
@@ -25,8 +27,11 @@
  // Linux >= 2.6.13
 #define HAVE_IOPRIO defined(__NR_ioprio_get) && defined(__NR_ioprio_set)
 
-// Linux >= 2.6.36 (supposedly)
-#define HAVE_PRLIMIT defined(PSUTIL_KERN_PRLIMIT) && defined(__NR_prlimit64)
+// Linux >= 2.6.36 (supposedly) and glibc >= 13
+#define HAVE_PRLIMIT \
+    (LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 36)) && \
+    (__GLIBC__ >= 2 && __GLIBC_MINOR__ >= 13) && \
+    defined(__NR_prlimit64)
 
 #if HAVE_PRLIMIT
     #define _FILE_OFFSET_BITS 64
